module ISTEX where

import ISTEX.Client
import Network.HTTP.Client (newManager)
import Network.HTTP.Client.TLS (tlsManagerSettings)
import Servant.Client

import Data.Text (Text)

runIstexAPIClient :: ClientM (Documents) -> IO (Either ClientError Documents)
runIstexAPIClient cmd = do
  manager' <- newManager tlsManagerSettings
  runClientM cmd (mkClientEnv manager' $ BaseUrl Https "api.istex.fr" 443 "document")

getMetadataWith :: Text -> Maybe Int -> IO (Either ClientError Documents)
getMetadataWith q n = do
  runIstexAPIClient $
    search
    (Just "author,title,abstract,publicationDate,refBibs")
    n
    (Just q)

type Progress = ScrollResponse -> IO ()
type ErrorHandler = ClientError -> IO ()

runIstexScrollAPIClient :: ClientM (ScrollResponse) -> IO (Either ClientError ScrollResponse)
runIstexScrollAPIClient cmd = do
  manager' <- newManager tlsManagerSettings
  runClientM cmd (mkClientEnv manager' $ BaseUrl Https "api.istex.fr" 443 "document")

maxScrollPages :: Int
maxScrollPages = 10

getMetadataScroll :: Text -> Text -> Maybe Text -> Int -> IO (Either ClientError Documents)
getMetadataScroll q scroll scrollId page = do
  if page >= maxScrollPages then
    pure $ Right mempty
  else do
    -- Prelude.putStrLn $ "[getMetadataScroll] scroll: " <> show scroll
    -- Prelude.putStrLn $ "[getMetadataScroll] scrollId: " <> show scrollId
    -- Prelude.putStrLn $ "[getMetadataScroll] page: " <> show page
    eRes <- runIstexScrollAPIClient $
      searchScroll
      (Just "author,title,abstract,publicationDate,refBibs")
      scroll
      (Just q)
      scrollId
    case eRes of
      Left err -> pure $ Left err
      Right res@(ScrollResponse { .. }) -> do
        eDocs <- getMetadataScroll q scroll (Just _scroll_scrollId) (page + 1)
        case eDocs of
          Left err -> pure $ Left err
          Right docs -> pure $ Right $ _scroll_documents <> docs
        --mappend _scroll_documents <*> getMetadataScroll q scroll (Just _scroll_scrollId) (page + 1)

getMetadataScrollProgress :: Text -> Text -> Maybe Text -> Progress -> ErrorHandler -> IO ()
getMetadataScrollProgress q scroll scrollId progress errorHandler = do
  eRes <- runIstexScrollAPIClient $
    searchScroll
    (Just "author,title,abstract,publicationDate,refBibs")
    scroll
    (Just q)
    scrollId
  case eRes of
    Left err -> errorHandler err
    Right res@(ScrollResponse { _scroll_noMoreScrollResults
                              , _scroll_nextScrollURI
                              , _scroll_scrollId }) -> do
      progress res
      getMetadataScrollProgress q scroll (Just _scroll_scrollId) progress errorHandler

type Query = Text

runTest :: Query -> Maybe Int -> IO ()
runTest query n = do
  res <- getMetadataWith query n
  case res of
    (Left err)  -> print $ show err
    (Right val) -> do
      print val


